class User < ApplicationRecord
  # Connects this user object to Hydra behaviors.
  include Hydra::User
  # Connects this user object to Role-management behaviors.
  include Hydra::RoleManagement::UserRoles

  include RDMSRoles
  # Connects this user object to Hyrax behaviors.
  include Hyrax::User
  include Hyrax::UserUsageStats



  if Blacklight::Utils.needs_attr_accessible?
    attr_accessible :email, :password, :password_confirmation
  end
  # Connects this user object to Blacklights Bookmarks.
  include Blacklight::User
  # Include default devise modules. Others available are:
  # :registerable, :recoverable, :confirmable, :lockable, :timeoutable, :trackable
  # Did not want option to sign_up or click on forgot_password.
  devise :database_authenticatable, :rememberable, :validatable,
         :omniauthable, :omniauth_providers => [:saml, :orcid]

  # Method added by Blacklight; Blacklight uses #to_s on your
  # user class to get a user-displayable login/identifier for
  # the account.
  def to_s
    email
  end

  def admin?
    roles.where(name: "admin").any?
  end

  def crc_manager_for?(group_id)
    group = Collection.find(group_id) rescue nil

    return false unless group

    roles.where(name: "#{group.collection_type.title.parameterize(separator: '_')}_manager").any?
  end

  def group_manager_for?(group_id)
    group = Collection.find(group_id) rescue nil

    return false unless group

    roles.where(name: "#{group.collection_type.title.parameterize(separator: '_')}_#{group.id}_manager").any?
  end

  def group_member_for?(group_id)
    group = Collection.find(group_id) rescue nil

    return false unless group

    roles.where(name: "#{group.collection_type.title.parameterize(separator: '_')}_#{group.id}_member").any?
  end

  def publication_manager?
    roles.where(name: "publication_manager").any?
  end

  def self.from_omniauth_orcid(auth)
    if ENV.fetch('ORCID_RESTRICT_AUTHORIZATION', 'true') == 'true'
      full_orcid = auth.uid
      full_orcid = "https://orcid.org/#{auth.uid}" unless auth.uid.start_with?("https://orcid.org/")
      user = User.where( provider: auth.provider, uid: auth.uid).or(User.where(orcid: full_orcid)).first
    else
      user = User.find_or_create_by(provider: auth.provider, uid: auth.uid)
    end

    return nil unless user

    user.orcid = auth.uid
    user.provider = auth.provider
    user.uid = auth.uid
    if ENV.fetch('ORCID_RESTRICT_AUTHORIZATION', 'true') == 'false'
      user.display_name = auth.info.name
      user.email = auth.info.email || "#{auth.uid}@#{auth.provider.to_s}"
      user.password = SecureRandom.random_bytes(32)
    end
    user.save!
    user
    # If you are using confirmable and the provider(s) you use validate emails,
    # uncomment the line below to skip the confirmation emails.
    # user.skip_confirmation!
  end

  def self.from_omniauth_saml(auth)
    # saml provider
    user = nil
    attributes = {
      uid: auth.extra.raw_info.attributes.fetch(ENV['SAML_ATTRIBUTE_UID'], [auth.uid]).first,
      name: auth.extra.raw_info.attributes.fetch(ENV['SAML_ATTRIBUTE_NAME'], [auth.uid]).first,
      email: auth.extra.raw_info.attributes.fetch(ENV['SAML_ATTRIBUTE_EMAIL'], ["#{auth.uid}@#{auth.provider.to_s}"]).first,
      role: auth.extra.raw_info.attributes.fetch(ENV['SAML_ATTRIBUTE_ROLE'], []),
    }
    if ENV.fetch('SAML_RESTRICT_AUTHORIZATION', 'true') == 'true'
      user = User.find_or_create_by(provider: auth.provider, uid: attributes[:uid]) if attributes[:role].map(&:downcase).include?('staff')
    else
      user = User.find_or_create_by(provider: auth.provider, uid: attributes[:uid])
    end

    return nil unless user

    user.provider = auth.provider
    user.uid = attributes[:uid]
    user.display_name = attributes[:name]
    user.email = attributes[:email]
    user.password = SecureRandom.random_bytes(32)

    user.save!
    user
    # If you are using confirmable and the provider(s) you use validate emails,
    # uncomment the line below to skip the confirmation emails.
    # user.skip_confirmation!
  end

  def can_tombstone?(presenter)
    self.admin?
  end

  def is_depositor_for?(presenter)
    presenter.depositor == self.user_key
  end

end
