# frozen_string_literal: true
class RegisterDoiWorker
  include Sidekiq::Worker 
  sidekiq_options retry: 0

  def perform(work_id, work_type)

    @dataset = work_type.constantize.find(work_id)

    url = URI(ENV['DOI_URL'])

    http = Net::HTTP.new(url.host, url.port)
    http.use_ssl = true
    creators_hash = @dataset.complex_person.map do |person|
      { name: "#{person.last_name.first},#{person.first_name.first}" }
    end.compact

    request = Net::HTTP::Post.new(url)
    request['content-type'] = 'application/vnd.api+json'
    request.basic_auth ENV['DOI_USERNAME'], ENV['DOI_PASSWORD']

    body = {
      "data": {
        "id": "#{ENV['DOI_PREFIX']}/#{@dataset.id}",
        "type": 'dois',
        "attributes": {
          "doi": "#{ENV['DOI_PREFIX']}/#{@dataset.id}",
          "creators": creators_hash,
          "titles": [{ title: @dataset.title&.first }],
          "publisher": @dataset.publisher&.first || 'Ruhr-University Bochum',
          "publicationYear": Date.today.year,
          "types": {
            "resourceTypeGeneral": @dataset.resource_type&.first
          }
        }
      }
    }
    request.body = body.to_json
    response = http.request(request)

    raise Net::HTTPFatalError.new response.message, response unless response.is_a?(Net::HTTPSuccess)

    parsed_response = JSON.parse(response.body)

    attributes = parsed_response['data']['attributes']

    raise Net::HTTPFatalError.new "No Attributes Found", response unless attributes.present?

    complex_identifier = ComplexIdentifier.new(RDF::Node.new, ActiveTriples::Resource.new)
    complex_date = ComplexDate.new(RDF::Node.new, ActiveTriples::Resource.new)

    complex_identifier.identifier = attributes['doi']
    complex_identifier.scheme = 'DOI'

    complex_date.date = attributes['created'].to_date.strftime('%d/%m/%Y')
    complex_date.description = 'Published'

    @dataset.update(complex_date: [complex_date], complex_identifier: [complex_identifier])

    @dataset.save
  end
end