namespace :rdms do
  desc 'Add users to CRC 1280. usage: rdms:add_crc_users["users.json"]'
  task :"add_crc_users", [:seedfile] => :environment do |task, args|
    seedfile = args.seedfile
    unless args.seedfile.present?
      seedfile = Rails.root.join("seed","users.json")
    end

    if (File.exists?(seedfile))
      puts("Running hyrax to add users from seedfile: #{seedfile}")
    else
      abort("ERROR: missing seedfile for user setup: #{seedfile}")
    end

    seed = JSON.parse(File.read(seedfile))

    # make the requested users with their roles
    roles = {
      "CRC manager" => Role.where(name: "crc_1280_manager").first ,
      "CRC member" => Role.where(name: "crc_1280_member").first,
    }

    # Create users and assign roles
    seed["users"].each do |user|
      puts "Adding user #{user["name"]}"
      newUser = User.where(email: user["email"].downcase).first_or_create!(
        password: user["password"],
        display_name: user["name"],
        affiliation: user['affiliation'],
        orcid: user['orcid']
      )

      user['role'].each do |r|
        group = r.split(' ')[0]
        unless roles.include?(r)
          # Find the role name from group
          crc_collection_title = ENV.fetch('CRC_1280_COLLECTION', "CRC 1280")
          collection_type = Hyrax::CollectionType.find_by(title: crc_collection_title)
          collections = collection_type.collections.select{ |t| t.title == [group] }
          next unless collections.present?
          manager_role = "crc_1280_#{collections[0].id}_manager" # crc_1280_6t053f96k_manager
          member_role = "crc_1280_#{collections[0].id}_member" # crc_1280_6t053f96k_member
          roles["#{group} manager"] = Role.where(name:manager_role).first
          roles["#{group} member"] = Role.where(name:member_role).first
        end
        role = roles[r]
        role.users << newUser unless role.users.include?(newUser)
      end
    end

    # Save roles
    roles.each do|_name, role|
      role.save!
    end
  end
end
